package fishrungames.bashgid.core;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.net.URL;
import java.util.HashMap;

import fishrungames.bashgid.MainActivity;
import fishrungames.bashgid.R;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.util.Base64;
import android.util.Log;
import android.widget.ImageView;

public class ImageManager {

	static ImageManager instance = null;
	
	public static ImageManager getInstance()
	{
		if (instance == null)
		{
			instance = new ImageManager();
		}
		
		return instance;
	}
	
	
	HashMap<String, Integer> PredefinedImageId = new HashMap<String, Integer>();
	
	public ImageManager()
	{
		PredefinedImageId.put("R.drawable.news_image1", R.drawable.news_image1);
		PredefinedImageId.put("R.drawable.news_image2", R.drawable.news_image2);
		PredefinedImageId.put("R.drawable.news_image3", R.drawable.news_image3);
	}
	
	
	
	
	public void ApplyImageToImageView(ImageView imageView, String imageId)
	{
		if (imageId.startsWith("R.drawable."))
		{
			Integer resourceId = PredefinedImageId.get(imageId);
			
			if (resourceId == null)
			{
				Log.e("Error: ", "Image not added to PredefinedImageId: " + imageId);
				
				resourceId = R.drawable.transparent;
			}
			
			imageView.setImageResource(resourceId);
		}
		else
		{
			String fileName = ImageUrlToFileName(imageId);
			
			File f = MainActivity.getInstance().getFileStreamPath(fileName);
			
			if (f.exists())
			{
				Bitmap bitmap = BitmapFactory.decodeFile( MainActivity.getInstance().getFileStreamPath(fileName).getAbsolutePath());
				
				imageView.setImageBitmap(bitmap);
			}
			else
			{
				Log.e("Error: ", "Image requested but not downloaded: " + imageId);

				imageView.setImageResource(R.drawable.transparent);
			}
		}
	}

	
	public static String ImageUrlToFileName(String imageUrl)
	{
		String beforeExtension = imageUrl.substring(0, imageUrl.lastIndexOf("."));
		String extension = imageUrl.substring(imageUrl.lastIndexOf("."));
		
		return Base64.encodeToString(beforeExtension.getBytes(), Base64.DEFAULT).replace("\n", "") + extension;
	}
	
	public void DownloadImageIfNeeded(String imageUrl)
	{
		String fileName = ImageUrlToFileName(imageUrl);
		
		File f = MainActivity.getInstance().getFileStreamPath(fileName);
		
		if (f.exists())
		{
			return;
		}
		
		try {
	        InputStream is = (InputStream) new URL(imageUrl).getContent();
	        
	        FileOutputStream out = null;
	        
	        out = MainActivity.getInstance().openFileOutput(fileName, Context.MODE_PRIVATE);
	        
	        byte [] buf = new byte[1024];
	         
	        int numRead;
	        
	        while ( (numRead = is.read(buf) ) >= 0) {
	        	out.write(buf, 0, numRead);
	        }
	        
	        out.close();

		} catch (Exception e) {
	        e.printStackTrace();
	    }
	}
	
	public String GetImageLocalUrl(String imageUrl)
	{
		return "file://" + MainActivity.getInstance().getFilesDir().getAbsolutePath() + "/" + ImageUrlToFileName(imageUrl);
	}
	
	
	
}
