package fishrungames.bashgid.core;

import java.util.ArrayList;

import fishrungames.bashgid.MainActivity;

public class NewsManager {

	private static NewsManager instance = null;
	
	
	public static NewsManager GetInstance()
	{
		if (instance == null)
		{
			instance = new NewsManager();
		}
		
		return instance;
	}
	
	
	public static class NewsRecord
	{
		public String title;
		public String description;
		public String imageId;
		
		public NewsRecord(String title, String description, String imageId)
		{
			this.title = title;
			this.description = description;
			this.imageId = imageId;
		}
		
		NewsRecord(RssReader.Rss2Item rss2Item)
		{
			this.title = rss2Item.title;
			this.description = rss2Item.description;
			
			if ((rss2Item.rss2Enclosure != null) && (DownloadManager.MimeTypeIsImage(rss2Item.rss2Enclosure.mimeType)))
			{
				this.imageId = rss2Item.rss2Enclosure.url;
			}
			else
			{
				this.imageId = "R.drawable.transparent";
			}
		}
	}
	
	ArrayList<NewsRecord> newsRecord = new ArrayList<NewsRecord>();
	
	public RemoveCallbackInterface DownloadNews(NewsUpdatedCallbackInterface callback)
	{
		
		DownloadNewsRunnable downloadNewsRunnable = new DownloadNewsRunnable(callback);
		
		Thread thread = new Thread(downloadNewsRunnable);
		
		thread.start();
		
		return downloadNewsRunnable;
	}
	
	class DownloadNewsRunnable implements Runnable, RemoveCallbackInterface
	{
		
		NewsUpdatedCallbackInterface callback;
		
		public DownloadNewsRunnable(NewsUpdatedCallbackInterface callback)
		{
			this.callback = callback;
		}
		
		@Override
		public void RemoveCallback()
		{
			callback = null;
		}
		
		@Override
		public void run()
		{
			//String url = "http://www.bashinform.ru/rss/all.xml";
			
			String url = "http://www.bashedu.ru/rss.xml";
			
			String xmlCode = DownloadManager.getXmlFromUrl(url);
			
			if (xmlCode == null)
			{
				return;
			}
			
			//Remove BOM character if present
			if (xmlCode.startsWith("\uFEFF"))
			{
				xmlCode = xmlCode.substring(1);
		    }
			
			
			
			RssReader rssReader = new RssReader();
			
			rssReader.ReadRss(xmlCode);
			
			ArrayList<NewsRecord> localNewsRecord = new ArrayList<NewsRecord>();
			
			for (RssReader.Rss2Item rss2Item : rssReader.rss2ItemArr)
			{
				//Enclosure
				if (rss2Item.rss2Enclosure != null)
				{
					if (DownloadManager.MimeTypeIsImage(rss2Item.rss2Enclosure.mimeType))
					{
						ImageManager.getInstance().DownloadImageIfNeeded(rss2Item.rss2Enclosure.url);
					}
				}
				
				//And images inside text
				int i = rss2Item.description.indexOf("<img");
				while (i != -1)
				{
					int j = rss2Item.description.indexOf("src=\"", i);
					
					if (j != -1)
					{
						j += ("src=\"").length();
						
						int k = rss2Item.description.indexOf("\"", j);
						
						String imageUrl = rss2Item.description.substring(j, k);
						
						String fullImageUrl = imageUrl;
						
						if (!fullImageUrl.startsWith("http://"))
						{
							String homeFolder = DownloadManager.GetUrlFolder(url);
							
							fullImageUrl = homeFolder + fullImageUrl;
						}
						ImageManager.getInstance().DownloadImageIfNeeded(fullImageUrl);
						
						String fileName = ImageManager.getInstance().GetImageLocalUrl(fullImageUrl);
						
						rss2Item.description = rss2Item.description.replace("\"" +imageUrl + "\"" , "\"" +fileName + "\"");
					}
					
					 i = rss2Item.description.indexOf("<img", j);
				}
			}
			
			for (RssReader.Rss2Item rss2Item : rssReader.rss2ItemArr)
			{
				localNewsRecord.add(new NewsRecord(rss2Item));
			}
			
			//Xperimental -- need mutex right here
			newsRecord = localNewsRecord;
			
			MainActivity.getInstance().newsDataSource.replaceNews(newsRecord);
			
			//Xperimental -- need mutex here, too
			if (callback != null)
			{
				callback.OnNewsUpdated();
			}
		}
	}
	
	public static interface NewsUpdatedCallbackInterface
	{
		public void OnNewsUpdated();
	}
	
	public static interface RemoveCallbackInterface
	{
		public void RemoveCallback();
	}
	
	
	public NewsManager()
	{
		newsRecord = MainActivity.getInstance().newsDataSource.getNews();
		
	}
	
	
	public ArrayList<NewsRecord> getNews()
	{
		return newsRecord;
	}
	
	
	
	
}
